#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `seamm_ff_util` package."""

import seamm_ff_util  # noqa: F401


def test_angle_angle_explicit(pcff):
    """Test of bond_bond parameters, which should fine explicit ones"""

    expected = {
        "K": "3.5475",
        "reference": "1",
        "Theta10": "107.6600",
        "Theta20": "105.8500",
    }

    i = "h"
    j = "c"
    k = "h"
    l = "n+"  # noqa: E741
    ptype, key, form, parameters = pcff.angle_angle_parameters(i, j, k, l)
    assert ptype == "explicit"
    assert key == ("h", "c", "h", "n+")
    assert parameters == expected


def test_angle_angle_explicit_ljki(pcff):
    """known bond_bond parameters, ordered backwards"""
    i = "h"
    j = "c"
    k = "h"
    l = "n+"  # noqa: E741
    ptype, key, form, parameters = pcff.angle_angle_parameters(i, j, k, l)
    ptype2, key2, form, parameters2 = pcff.angle_angle_parameters(l, j, k, i)
    assert ptype2 == "explicit"
    assert key2 == ("n+", "c", "h", "h")
    assert parameters == parameters2


def test_angle_angle_equivalent(pcff):
    """Simple test of bond_bond parameters using equivalencies"""
    expected = {
        "K": "5.9863",
        "reference": "6",
        "Theta10": "116.0640",
        "Theta20": "116.0640",
    }

    i = "c5"
    j = "cp"
    k = "c_1"
    l = "c5"  # noqa: E741
    ptype, key, form, parameters = pcff.angle_angle_parameters(i, j, k, l)
    assert ptype == "equivalent"
    assert key == ("cp", "cp", "c_1", "cp")
    assert parameters == expected



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `seamm_ff_util` package."""

import seamm_ff_util  # noqa: F401


def test_angle_torsion_3_explicit(pcff):
    """Test of angle_torsion_3 parameters, which should find
    explicit ones"""

    expected = {
        "V1_L": "0.0492",
        "V1_R": "-1.6930",
        "V2_L": "0.7162",
        "V2_R": "-0.6252",
        "V3_L": "-0.2277",
        "V3_R": "-0.2148",
        "reference": "1",
        "Theta0_L": "108.4000",
        "Theta0_R": "110.7700",
    }

    i = "h"
    j = "c"
    k = "c"
    l = "c_0"  # noqa: E741
    ptype, key, form, parameters = pcff.angle_torsion_3_parameters(i, j, k, l)
    assert ptype == "explicit"
    assert key == ("h", "c", "c", "c_0")
    assert parameters == expected


def test_angle_torsion_3_explicit_kji(pcff):
    """known angle_torsion_3 parameters, ordered backwards"""

    expected = {
        "V1_L": "-1.6930",
        "V1_R": "0.0492",
        "V2_L": "-0.6252",
        "V2_R": "0.7162",
        "V3_L": "-0.2148",
        "V3_R": "-0.2277",
        "reference": "1",
        "Theta0_L": "108.4000",
        "Theta0_R": "110.7700",
    }

    i = "c_0"
    j = "c"
    k = "c"
    l = "h"  # noqa: E741
    ptype, key, form, parameters = pcff.angle_torsion_3_parameters(i, j, k, l)
    assert ptype == "explicit"
    assert key == ("c_0", "c", "c", "h")
    assert parameters == expected


def test_angle_torsion_3_equivalent(pcff):
    """Simple test of angle_torsion_3 parameters using equivalencies"""

    expected = {
        "V1_L": "4.6266",
        "V1_R": "0.2251",
        "V2_L": "0.1632",
        "V2_R": "0.6548",
        "V3_L": "0.0461",
        "V3_R": "0.1237",
        "reference": "1",
        "Theta0_L": "111.0000",
        "Theta0_R": "120.0500",
    }

    i = "h"
    j = "c"
    k = "c5"
    l = "c5"  # noqa: E741
    ptype, key, form, parameters = pcff.angle_torsion_3_parameters(i, j, k, l)
    assert ptype == "equivalent"
    assert key == ("h", "c", "cp", "cp")
    assert parameters == expected



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `seamm_ff_util` package."""

import seamm_ff_util  # noqa: F401


def test_bond_angle_explicit(pcff):
    """Test of bond_angle parameters, which should fine explicit ones"""

    expected = {
        "K12": "20.7540",
        "K23": "11.4210",
        "reference": "1",
        "R10": "1.5300",
        "R20": "1.1010",
    }

    i = "c"
    j = "c"
    k = "h"
    ptype, key, form, parameters = pcff.bond_angle_parameters(i, j, k)
    assert ptype == "explicit"
    assert key == ("c", "c", "h")
    assert parameters == expected


def test_bond_angle_explicit_kji(pcff):
    """known bond_angle parameters, ordered backwards"""

    expected = {
        "K12": "11.4210",
        "K23": "20.7540",
        "reference": "1",
        "R10": "1.5300",
        "R20": "1.1010",
    }
    i = "h"
    j = "c"
    k = "c"
    ptype, key, form, parameters = pcff.bond_angle_parameters(i, j, k)
    assert ptype == "explicit"
    assert key == ("h", "c", "c")
    assert parameters == expected


def test_bond_angle_equivalent(pcff):
    """Simple test of bond_angle parameters using equivalencies"""
    expected = {
        "K12": "20.7540",
        "K23": "11.4210",
        "reference": "1",
        "R10": "1.5300",
        "R20": "1.1010",
    }

    i = "c"
    j = "c1"
    k = "h"
    ptype, key, form, parameters = pcff.bond_angle_parameters(i, j, k)
    assert ptype == "equivalent"
    assert key == ("c", "c", "h")
    assert parameters == expected



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Fixtures for testing the 'seamm_ff_util' package."""

import pytest

from molsystem import SystemDB
from seamm_ff_util import Forcefield
from seamm_ff_util import FFAssigner


@pytest.fixture(scope="session")
def pcff():
    """A forcefield object initialized with PCFF"""
    pcff = Forcefield("data/pcff2018.frc")
    pcff.initialize_biosym_forcefield()
    return pcff


@pytest.fixture(scope="session")
def pcff_assigner(pcff):
    """A forcefield object initialized with PCFF"""
    pcff_assigner = FFAssigner(pcff)
    return pcff_assigner


@pytest.fixture()
def configuration():
    """Create a system db with no systems."""
    db = SystemDB(filename="file:seamm_db?mode=memory&cache=shared")
    system = db.create_system(name="default")
    configuration = system.create_configuration(name="default")

    yield configuration

    db.close()
    try:
        del db
    except:  # noqa: E722
        print("Caught error deleting the database")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `seamm_ff_util` package."""

import seamm_ff_util  # noqa: F401


def test_oop_explicit(pcff):
    """Test of out-of-plane parameters, which should find explicit ones"""

    expected = {"Chi0": "0.0000", "K": "10.8102", "reference": "1"}

    i = "h"
    j = "cp"
    k = "np"
    l = "op"  # noqa: E741
    ptype, key, form, parameters = pcff.oop_parameters(i, j, k, l)
    assert ptype == "explicit"
    assert key == ("h", "cp", "np", "op")
    assert parameters == expected


def test_oop_explicit_lkji(pcff):
    """known out-of-plane parameters, ordered backwards"""
    i = "h"
    j = "cp"
    k = "np"
    l = "op"  # noqa: E741
    ptype, key, form, parameters = pcff.oop_parameters(i, j, k, l)
    ptype2, key2, form, parameters2 = pcff.oop_parameters(l, j, k, i)
    assert ptype2 == "explicit"
    assert key == ("h", "cp", "np", "op")
    assert parameters == parameters2


def test_oop_equivalent(pcff):
    """Simple test of out-of-plane parameters using equivalencies"""
    expected = {"Chi0": "0.0000", "K": "10.8102", "reference": "1"}

    i = "hp"
    j = "c5"
    k = "np"
    l = "op"  # noqa: E741
    ptype, key, form, parameters = pcff.oop_parameters(i, j, k, l)
    assert ptype == "equivalent"
    assert key == ("h", "cp", "np", "op")
    assert parameters == expected


def test_oop_auto(pcff):
    """test of out-of-plane parameters using automatic parameters"""
    expected = {"Chi0": "0.0000", "K": "36.0000", "reference": "1"}

    i = "hp"
    j = "c_0"
    k = "co"
    l = "br"  # noqa: E741
    ptype, key, form, parameters = pcff.oop_parameters(i, j, k, l)
    assert ptype == "automatic"
    assert key == ("*", "c'_", "*", "*")
    assert parameters == expected



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `seamm_ff_util` package."""

import seamm_ff_util  # noqa: F401


def test_torsion_explicit(pcff):
    """Test of torsion parameters, which should find explicit ones"""

    expected = {
        "Phi0_1": "0.0",
        "Phi0_2": "0.0",
        "Phi0_3": "0.0",
        "V1": "0.0000",
        "V2": "0.0000",
        "V3": "-0.2000",
        "reference": "8",
    }

    i = "h"
    j = "c"
    k = "c"
    l = "br"  # noqa: E741
    ptype, key, form, parameters = pcff.torsion_parameters(i, j, k, l)
    assert ptype == "explicit"
    assert key == ("br", "c", "c", "h")
    assert parameters == expected


def test_torsion_explicit_lkji(pcff):
    """known torsion parameters, ordered backwards"""
    i = "h"
    j = "c"
    k = "c"
    l = "br"  # noqa: E741
    ptype, key, form, parameters = pcff.torsion_parameters(i, j, k, l)
    ptype2, key2, form, parameters2 = pcff.torsion_parameters(l, k, j, i)
    assert ptype2 == "explicit"
    assert key2 == ("br", "c", "c", "h")
    assert parameters == parameters2


def test_torsion_equivalent(pcff):
    """Simple test of torsion parameters using equivalencies"""
    expected = {
        "Phi0_1": "0.0",
        "Phi0_2": "0.0",
        "Phi0_3": "0.0",
        "V1": "0.0000",
        "V2": "1.8769",
        "V3": "0.0000",
        "reference": "1",
    }

    i = "hp"
    j = "c5"
    k = "c5"
    l = "hp"  # noqa: E741
    ptype, key, form, parameters = pcff.torsion_parameters(i, j, k, l)
    assert ptype == "equivalent"
    assert key == ("h", "cp", "cp", "h")
    assert parameters == expected


def test_torsion_auto(pcff):
    """test of torsion parameters using automatic parameters"""
    expected = {"KPhi": "3.0000", "Phi0": "180.0000", "n": "2", "reference": "2"}

    i = "hp"
    j = "c5"
    k = "c5"
    l = "br"  # noqa: E741
    ptype, key, form, parameters = pcff.torsion_parameters(i, j, k, l)
    assert ptype == "automatic"
    assert key == ("*", "cp_", "cp_", "*")
    assert parameters == expected



# -*- coding: utf-8 -*-

"""Metadata for describing terms in focefields.
"""

metadata = {
    "charges": {
        "equation": ["I"],
        "constants": [
            ("Q", "e"),
        ],
        "topology": {
            "type": "atomic charge",
            "n_atoms": 1,
            "symmetry": "none",
            "fill": 0,
            "flip": 0,
        },
    },
    "shell-model": {
        "equation": ["I"],
        "constants": [
            ("Q", "e"),
            ("Y", "e"),
            ("k", "kcal/mol/Å^2"),
        ],
        "topology": {
            "type": "shell model",
            "n_atoms": 1,
            "symmetry": "none",
            "fill": 0,
            "flip": 0,
        },
    },
    "bond_increments": {
        "equation": ["delta"],
        "constants": [
            ("deltaij", "e"),
            ("deltaji", "e"),
        ],
        "topology": {
            "type": "bond charge increment",
            "n_atoms": 2,
            "symmetry": "like_bond",
            "fill": 0,
            "flip": 1,
        },
    },
    "quadratic_bond": {
        "equation": ["K2*(R-R0)^2"],
        "constants": [
            ("R0", "Å"),
            ("K2", "kcal/mol/Å^2"),
        ],
        "topology": {
            "type": "bond",
            "n_atoms": 2,
            "symmetry": "like_bond",
            "fill": 0,
            "flip": 0,
        },
    },
    "quartic_bond": {
        "equation": ["K2*(R-R0)^2 + K3*(R-R0)^3 + K4*(R-R0)^4"],
        "constants": [
            ("R0", "Å"),
            ("K2", "kcal/mol/Å^2"),
            ("K3", "kcal/mol/Å^3"),
            ("K4", "kcal/mol/Å^4"),
        ],
        "topology": {
            "type": "bond",
            "n_atoms": 2,
            "symmetry": "like_bond",
            "fill": 0,
            "flip": 0,
        },
    },
    "quadratic_angle": {
        "equation": ["K2*(Theta-Theta0)^2"],
        "constants": [
            ("Theta0", "degree"),
            ("K2", "kcal/mol/radian^2"),
        ],
        "topology": {
            "type": "angle",
            "n_atoms": 3,
            "symmetry": "like_angle",
            "fill": 0,
            "flip": 0,
        },
    },
    "quartic_angle": {
        "equation": [
            ("K2*(Theta-Theta0)^2 + K3*(Theta-Theta0)^3" "+ K4*(Theta-Theta0)^4")
        ],
        "constants": [
            ("Theta0", "degree"),
            ("K2", "kcal/mol/radian^2"),
            ("K3", "kcal/mol/radian^3"),
            ("K4", "kcal/mol/radian^4"),
        ],
        "topology": {
            "type": "angle",
            "n_atoms": 3,
            "symmetry": "like_angle",
            "fill": 0,
            "flip": 0,
        },
    },
    "simple_fourier_angle": {
        "equation": ["K*[1-cos(n*Theta)]"],
        "constants": [
            ("K", "kcal/mol"),
            ("n", ""),
        ],
        "topology": {
            "type": "angle",
            "n_atoms": 3,
            "symmetry": "like_angle",
            "fill": 0,
            "flip": 0,
        },
    },
    "tabulated_angle": {
        "constants": [
            ("Eqn", "", str),
            ("K", "kcal/mol", float),
            ("n", "", int),
            ("Rb", "Å", float),
            ("A", "kcal/mol*Å^12", float),
            ("zero-shift", "degree", float),
        ],
        "topology": {
            "type": "angle",
            "n_atoms": 3,
            "symmetry": "like_angle",
            "fill": 0,
            "flip": 0,
        },
    },
    "torsion_1": {
        "equation": ["KPhi * [1 + cos(n*Phi - Phi0)]"],
        "constants": [
            ("KPhi", "kcal/mol"),
            ("n", ""),
            ("Phi0", "degree"),
        ],
        "topology": {
            "type": "torsion",
            "n_atoms": 4,
            "symmetry": "like_torsion",
            "fill": 0,
            "flip": 0,
        },
    },
    "torsion_3": {
        "equation": [
            (
                "V1 * [1 + cos(Phi - Phi0_1)]"
                " + V2 * [1 + cos(2*Phi - Phi0_2)]"
                " + V3 * [1 + cos(3*Phi - Phi0_3)]"
            )
        ],
        "constants": [
            ("V1", "kcal/mol"),
            ("Phi0_1", "degree"),
            ("V2", "kcal/mol"),
            ("Phi0_2", "degree"),
            ("V3", "kcal/mol"),
            ("Phi0_3", "degree"),
        ],
        "topology": {
            "type": "torsion",
            "n_atoms": 4,
            "symmetry": "like_torsion",
            "fill": 0,
            "flip": 0,
        },
    },
    "torsion_opls": {
        "equation": [
            (
                "  1/2 * V1 * [1 + cos(Phi)]"
                "+ 1/2 * V2 * [1 - cos(Phi)]"
                "+ 1/2 * V3 * [1 + cos(Phi)]"
                "+ 1/2 * V4 * [1 - cos(Phi)]"
            )
        ],
        "constants": [
            ("V1", "kcal/mol"),
            ("V2", "kcal/mol"),
            ("V3", "kcal/mol"),
            ("V4", "kcal/mol"),
        ],
        "topology": {
            "type": "torsion",
            "n_atoms": 4,
            "symmetry": "like_torsion",
            "fill": 0,
            "flip": 0,
        },
    },
    "wilson_out_of_plane": {
        "equation": ["K*(Chi - Chi0)^2"],
        "constants": [
            ("K", "kcal/mol/radian^2"),
            ("Chi0", "degree"),
        ],
        "topology": {
            "type": "out-of-plane",
            "n_atoms": 4,
            "symmetry": "like_oop",
            "fill": 0,
            "flip": 0,
        },
    },
    "improper_opls": {
        "equation": ["1/2 * V2 * [1 - cos(Phi)]"],
        "constants": [
            ("V2", "kcal/mol"),
        ],
        "topology": {
            "type": "out-of-plane",
            "n_atoms": 4,
            "symmetry": "like_improper",
            "fill": 0,
            "flip": 0,
        },
    },
    "nonbond(9-6)": {
        "equation": [
            "eps(ij) [2(r(ij)*/r(ij))**9 - 3(r(ij)*/r(ij))**6]",
            "r(ij) = [(r(i)**6 + r(j)**6))/2]**(1/6)",
            (
                "eps(ij) = 2 * sqrt(eps(i) * eps(j)) * "
                "r(i)^3 * r(j)^3/[r(i)^6 + r(j)^6]"
            ),
        ],
        "constants": [("rmin", "Å"), ("eps", "kcal/mol")],
        "topology": {
            "form": "rmin-eps",
            "type": "pair",
            "subtype": "LJ 6-9",
            "n_atoms": 1,
            "symmetry": "none",
            "fill": 0,
            "flip": 0,
        },
    },
    "nonbond(12-6)": {
        "equation": [
            "E = 4 * eps * [(sigma/r)**12 - (sigma/r)**6]",
            "E = eps * [(rmin/r)**12 - (rmin/r)**6]",
            "E = A/r**12 - B/r**6",
            "rmin = 2**1/6 * sigma ",
            "sigma = rmin / 2**1/6",
            "A = 4 * eps * sigma**12",
            "B = 4 * eps * sigma**6",
            "sigma = (A/B)**1/6",
            "eps = B**2/(4*A)",
        ],
        "constants": [("sigma", "Å"), ("eps", "kcal/mol")],
        "topology": {
            "form": "sigma-eps",
            "type": "pair",
            "subtype": "LJ 12-6",
            "n_atoms": 1,
            "symmetry": "none",
            "fill": 0,
            "flip": 0,
        },
    },
    "buckingham": {
        "equation": ["E = A*exp(r/rho) - C/r**6"],
        "constants": [
            ("A", "kcal/mol"),
            ("rho", "Å"),
            ("C", "kcal/mol*Å**6"),
            ("cutoff", "Å"),
        ],
        "topology": {
            "type": "pair",
            "subtype": "LJ exp-6",
            "n_atoms": 2,
            "symmetry": "like_bond",
            "fill": 0,
            "flip": 0,
        },
    },
    "bond-bond": {
        "equation": ["K*(R-R0)*(R'-R0')"],
        "constants": [("K", "kcal/mol/Å^2")],
        "topology": {
            "type": "bond-bond",
            "n_atoms": 3,
            "symmetry": "like_angle",
            "fill": 0,
            "flip": 0,
        },
    },
    "bond-bond_1_3": {
        "equation": ["K*(R-R0)*(R'-R0')"],
        "constants": [("K", "kcal/mol/Å^2")],
        "topology": {
            "type": "1,3 bond-bond",
            "n_atoms": 4,
            "symmetry": "like_torsion",
            "fill": 0,
            "flip": 0,
        },
    },
    "bond-angle": {
        "equation": ["K*(R-R0)*(Theta-Theta0)"],
        "constants": [
            ("K12", "kcal/mol/Å/radian"),
            ("K23", "kcal/mol/Å/radian"),
        ],
        "topology": {
            "type": "bond-angle",
            "n_atoms": 3,
            "symmetry": "like_angle",
            "fill": 1,
            "flip": 1,
        },
    },
    "angle-angle": {
        "equation": ["K*(Theta-Theta0)*(Theta'-Theta0')"],
        "constants": [("K", "kcal/mol/Å/radian")],
        "topology": {
            "type": "angle-angle",
            "n_atoms": 4,
            "symmetry": "like_angle-angle",
            "fill": 0,
            "flip": 0,
        },
    },
    "end_bond-torsion_3": {
        "equation": [
            (
                "(R_L - R0_L) * (V1_L * [1 + cos(Phi - Phi0_1)]"
                " + V2_L * [1 + cos(2*Phi - Phi0_2)]"
                " + V3_L * [1 + cos(3*Phi - Phi0_3)])"
            ),
            (
                "(R_R - R0_R) * (V1_R * [1 + cos(Phi - Phi0_1)]"
                " + V2_R * [1 + cos(2*Phi - Phi0_2)]"
                " + V3_R * [1 + cos(3*Phi - Phi0_3)])"
            ),
        ],
        "constants": [
            ("V1_L", "kcal/mol"),
            ("V2_L", "kcal/mol"),
            ("V3_L", "kcal/mol"),
            ("V1_R", "kcal/mol"),
            ("V2_R", "kcal/mol"),
            ("V3_R", "kcal/mol"),
        ],
        "topology": {
            "type": "torsion-end bond",
            "n_atoms": 4,
            "symmetry": "like_torsion",
            "fill": 3,
            "flip": 3,
        },
    },
    "middle_bond-torsion_3": {
        "equation": [
            (
                "(R_M - R0_M) * (V1 * [1 + cos(Phi - Phi0_1)]"
                " + V2 * [1 + cos(2*Phi - Phi0_2)]"
                " + V3 * [1 + cos(3*Phi - Phi0_3)])"
            )
        ],
        "constants": [
            ("V1", "kcal/mol"),
            ("V2", "kcal/mol"),
            ("V3", "kcal/mol"),
        ],
        "topology": {
            "type": "torsion-middle bond",
            "n_atoms": 4,
            "symmetry": "like_torsion",
            "fill": 0,
            "flip": 0,
        },
    },
    "angle-torsion_3": {
        "equation": [
            (
                "(Theta_L - Theta0_L)"
                "* (V1_L * [1 + cos(Phi - Phi0_1)]"
                " + V2_L * [1 + cos(2*Phi - Phi0_2)]"
                " + V3_L * [1 + cos(3*Phi - Phi0_3)])"
            ),
            (
                "(Theta_R - Theta0_R)"
                " * (V1_R * [1 + cos(Phi - Phi0_1)]"
                " + V2_R * [1 + cos(2*Phi - Phi0_2)]"
                " + V3_R * [1 + cos(3*Phi - Phi0_3)])"
            ),
        ],
        "constants": [
            ("V1_L", "kcal/mol"),
            ("V2_L", "kcal/mol"),
            ("V3_L", "kcal/mol"),
            ("V1_R", "kcal/mol"),
            ("V2_R", "kcal/mol"),
            ("V3_R", "kcal/mol"),
        ],
        "topology": {
            "type": "torsion-angle",
            "n_atoms": 4,
            "symmetry": "like_torsion",
            "fill": 3,
            "flip": 3,
        },
    },
    "angle-angle-torsion_1": {
        "equation": [
            ("K * (Theta_L - Theta0_L) * (Theta_R - Theta0_R) * " "(Phi - Phi0_1)")
        ],
        "constants": [("K", "kcal/mol/degree^2/degree")],
        "topology": {
            "type": "angle-torsion-angle",
            "n_atoms": 4,
            "symmetry": "like_torsion",
            "fill": 0,
            "flip": 0,
        },
    },
    "torsion-torsion_1": {
        "equation": ["K * cos(Phi_L) * cos(Phi_R)"],
        "constants": [("K", "kcal/mol")],
        "topology": {
            "type": "torsion-torsion",
            "n_atoms": 5,
            "symmetry": "like_torsion-torsion",
            "fill": 0,
            "flip": 0,
        },
    },
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `seamm_ff_util` package."""

import seamm_ff_util  # noqa: F401


def test_bond_explicit(pcff):
    """Simple test of known bond parameters"""
    expected = {
        "K2": "345.0000",
        "K3": "-691.8900",
        "K4": "844.6000",
        "R0": "1.1010",
        "reference": "8",
    }

    i = "c"
    j = "h"
    ptype, key, form, parameters = pcff.bond_parameters(i, j)
    assert ptype == "explicit"
    assert key == ("c", "h")
    assert parameters == expected


def test_bond_explicit_ji(pcff):
    """Simple test of known bond parameters, ordered backwards"""
    i = "c"
    j = "h"
    ptype, key, form, parameters = pcff.bond_parameters(i, j)
    ptype2, key2, form, parameters2 = pcff.bond_parameters(j, i)
    assert ptype2 == "explicit"
    assert key2 == ("c", "h")
    assert parameters == parameters2


def test_bond_equivalent(pcff):
    """Simple test of bond parameters using equivalencies"""
    expected = {
        "K2": "372.8251",
        "K3": "-803.4526",
        "K4": "894.3173",
        "R0": "1.0982",
        "reference": "8",
    }

    i = "c5"
    j = "hp"
    ptype, key, form, parameters = pcff.bond_parameters(i, j)
    assert ptype == "equivalent"
    assert key == ("cp", "h")
    assert parameters == expected


def test_bond_auto(pcff):
    """Simple test of bond parameters using automatic parameters"""
    expected = {"K2": "223.6000", "R0": "1.9200", "reference": "2"}

    i = "c5"
    j = "br"
    ptype, key, form, parameters = pcff.bond_parameters(i, j)
    assert ptype == "automatic"
    assert key == ("br_", "cp_")
    assert parameters == expected



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `forcefield` package."""

import json

import seamm_ff_util  # noqa: F401


def test_angle_explicit(pcff):
    """Test of angle parameters, which should find explicit ones"""

    expected = {
        "K2": "46.0608",
        "K3": "0.0000",
        "K4": "0.0000",
        "Theta0": "106.9999",
        "reference": "8",
    }

    i = "h"
    j = "c"
    k = "br"
    ptype, key, form, parameters = pcff.angle_parameters(i, j, k)
    assert ptype == "explicit"
    assert key == ("br", "c", "h")
    if parameters != expected:
        print("parameters:")
        print(json.dumps(parameters, indent=4))
    assert parameters == expected


def test_angle_explicit_kji(pcff):
    """known angle parameters, ordered backwards"""
    i = "h"
    j = "c"
    k = "br"
    ptype, key, form, parameters = pcff.angle_parameters(i, j, k)
    ptype2, key2, form, parameters2 = pcff.angle_parameters(k, j, i)
    assert ptype2 == "explicit"
    assert key2 == ("br", "c", "h")
    assert parameters == parameters2


def test_angle_equivalent(pcff):
    """Simple test of angle parameters using equivalencies"""
    expected = {
        "K2": "35.1558",
        "K3": "-12.4682",
        "K4": "0.0000",
        "Theta0": "117.9400",
        "reference": "1",
    }

    i = "c5"
    j = "c5"
    k = "hp"
    ptype, key, form, parameters = pcff.angle_parameters(i, j, k)
    assert ptype == "equivalent"
    assert key == ("cp", "cp", "h")
    assert parameters == expected


def test_angle_auto(pcff):
    """test of angle parameters using automatic parameters"""
    expected = {"K2": "80.0000", "Theta0": "120.0000", "reference": "2"}

    i = "c5"
    j = "c5"
    k = "br"
    ptype, key, form, parameters = pcff.angle_parameters(i, j, k)
    assert ptype == "automatic"
    assert key == ("*7", "cp_", "c_")
    assert parameters == expected



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `seamm_ff_util` package."""

import seamm_ff_util  # noqa: F401


def test_angle_angle_torsion_1_explicit(pcff):
    """Test of angle_angle_torsion_1 parameters, which should find
    explicit ones"""

    expected = {
        "K": "-14.3155",
        "reference": "1",
        "Theta0_L": "110.7700",
        "Theta0_R": "108.4000",
    }

    i = "h"
    j = "c"
    k = "c"
    l = "c_0"  # noqa: E741
    ptype, key, form, parameters = pcff.angle_angle_torsion_1_parameters(i, j, k, l)
    assert ptype == "explicit"
    assert key == ("c_0", "c", "c", "h")
    assert parameters == expected


def test_angle_angle_torsion_1_explicit_lkji(pcff):
    """known angle_angle_torsion_1 parameters, ordered backwards"""

    expected = {
        "K": "-14.3155",
        "reference": "1",
        "Theta0_L": "108.4000",
        "Theta0_R": "110.7700",
    }

    i = "c_0"
    j = "c"
    k = "c"
    l = "h"  # noqa: E741
    ptype, key, form, parameters = pcff.angle_angle_torsion_1_parameters(i, j, k, l)
    assert ptype == "explicit"
    assert key == ("c_0", "c", "c", "h")
    assert parameters == expected


def test_angle_angle_torsion_1_equivalent(pcff):
    """Simple test of angle_angle_torsion_1 parameters using equivalencies"""

    expected = {
        "K": "-5.8888",
        "reference": "1",
        "Theta0_L": "111.0000",
        "Theta0_R": "120.0500",
    }

    i = "h"
    j = "c"
    k = "c5"
    l = "c5"  # noqa: E741
    ptype, key, form, parameters = pcff.angle_angle_torsion_1_parameters(i, j, k, l)
    assert ptype == "equivalent"
    assert key == ("h", "c", "cp", "cp")
    assert parameters == expected



# -*- coding: utf-8 -*-

"""Main class for handling forcefields"""

import logging
import rdkit
import rdkit.Chem
import rdkit.Chem.Draw
import rdkit.Chem.AllChem

logger = logging.getLogger(__name__)
# logger.setLevel("DEBUG")


class FFAssigner(object):
    def __init__(self, forcefield):
        """Handle the assignment of the forcefield to the structure

        This class is closely related to the Forcefield class, but
        separated from it due to the dependencies it carries along,
        coupled with the fact that it is not needed in some
        computations where the forcefield itself is.
        """

        self.forcefield = forcefield

    def assign(self, configuration):
        """Assign the atom types to the structure using SMARTS templates"""
        molecule = configuration.to_RDKMol()

        n_atoms = configuration.n_atoms

        atom_types = ["?"] * n_atoms
        templates = self.forcefield.get_templates()
        for atom_type in templates:
            template = templates[atom_type]
            for smarts in template["smarts"]:
                pattern = rdkit.Chem.MolFromSmarts(smarts)

                ind_map = {}
                for atom in pattern.GetAtoms():
                    map_num = atom.GetAtomMapNum()
                    if map_num:
                        ind_map[map_num - 1] = atom.GetIdx()
                map_list = [ind_map[x] for x in sorted(ind_map)]

                matches = molecule.GetSubstructMatches(pattern, maxMatches=6 * n_atoms)
                logger.debug(atom_type + ": ")
                if len(matches) > 0:
                    for match in matches:
                        atom_ids = [match[x] for x in map_list]
                        for x in atom_ids:
                            atom_types[x] = atom_type
                        tmp = [str(x) for x in atom_ids]
                        logger.debug("\t" + ", ".join(tmp))

        i = 0
        untyped = []
        for atom, atom_type in zip(molecule.GetAtoms(), atom_types):
            if atom_type == "?":
                untyped.append(i)
            logger.debug("{}: {}".format(atom.GetSymbol(), atom_type))
            i += 1

        if len(untyped) > 0:
            logger.warning(
                "The forcefield does not have atom types for"
                " the molecule!. See missing_atom_types.png"
                " for more detail."
            )
            rdkit.Chem.AllChem.Compute2DCoords(molecule)
            img = rdkit.Chem.Draw.MolToImage(
                molecule,
                size=(1000, 1000),
                highlightAtoms=untyped,
                highlightColor=(0, 1, 0),
            )
            img.save("missing_atom_types.png")
        else:
            logger.info("The molecule was successfully atom-typed")

        return atom_types



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `seamm_ff_util` package."""

import seamm_ff_util  # noqa: F401


def test_bond_bond_1_3_explicit(pcff):
    """Test of bond_bond parameters, which should fine explicit ones"""

    expected = {"K": "-3.4826", "reference": "1", "R10": "1.1010", "R30": "1.4170"}

    i = "h"
    j = "c"
    k = "cp"
    l = "cp"  # noqa: E741
    ptype, key, form, parameters = pcff.bond_bond_1_3_parameters(i, j, k, l)
    assert ptype == "explicit"
    assert key == ("h", "c", "cp", "cp")
    assert parameters == expected


def test_bond_bond_1_3_explicit_kji(pcff):
    """known bond_bond parameters, ordered backwards"""
    i = "h"
    j = "c"
    k = "cp"
    l = "cp"  # noqa: E741
    ptype, key, form, parameters = pcff.bond_bond_1_3_parameters(i, j, k, l)
    ptype2, key2, form, parameters2 = pcff.bond_bond_1_3_parameters(l, k, j, i)
    assert ptype2 == "explicit"
    assert key2 == ("h", "c", "cp", "cp")
    assert parameters == parameters2


def test_bond_bond_1_3_equivalent(pcff):
    """Simple test of bond_bond parameters using equivalencies"""
    expected = {"K": "-3.4826", "reference": "1", "R10": "1.1010", "R30": "1.4170"}

    i = "h"
    j = "c1"
    k = "cp"
    l = "cp"  # noqa: E741
    ptype, key, form, parameters = pcff.bond_bond_1_3_parameters(i, j, k, l)
    assert ptype == "equivalent"
    assert key == ("h", "c", "cp", "cp")
    assert parameters == expected



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `seamm_ff_util` package."""

import seamm_ff_util  # noqa: F401


def test_nonbond_explicit(pcff):
    """Simple test of known nonbond parameters"""
    expected = {"reference": "1", "rmin": 2.995, "eps": 0.02}

    i = "h"
    ptype, key, form, parameters = pcff.nonbond_parameters(i, form="nonbond(9-6)")
    assert ptype == "explicit"
    assert key == ("h",)
    assert parameters == expected


def test_nonbond_equivalent(pcff):
    """Simple test of nonbond parameters using equivalencies"""
    expected = {"reference": "1", "rmin": 4.01, "eps": 0.064}

    i = "c5"
    ptype, key, form, parameters = pcff.nonbond_parameters(i, form="nonbond(9-6)")
    assert ptype == "equivalent"
    assert key == ("cp",)
    assert parameters == expected



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `seamm_ff_util` package."""

import seamm_ff_util  # noqa: F401


def test_middle_bond_torsion_3_explicit(pcff):
    """Test of middle_bond_torsion_3 parameters, which should find
    explicit ones"""

    expected = {
        "V1": "-10.0179",
        "V2": "-2.8145",
        "V3": "0.1665",
        "reference": "1",
        "R0": "1.5300",
    }

    i = "h"
    j = "c"
    k = "c"
    l = "c_0"  # noqa: E741
    ptype, key, form, parameters = pcff.middle_bond_torsion_3_parameters(i, j, k, l)
    assert ptype == "explicit"
    assert key == ("c_0", "c", "c", "h")
    assert parameters == expected


def test_middle_bond_torsion_3_explicit_lkji(pcff):
    """known middle_bond_torsion_3 parameters, ordered backwards"""

    expected = {
        "V1": "-10.0179",
        "V2": "-2.8145",
        "V3": "0.1665",
        "reference": "1",
        "R0": "1.5300",
    }

    i = "c_0"
    j = "c"
    k = "c"
    l = "h"  # noqa: E741
    ptype, key, form, parameters = pcff.middle_bond_torsion_3_parameters(i, j, k, l)
    assert ptype == "explicit"
    assert key == ("c_0", "c", "c", "h")
    assert parameters == expected


def test_middle_bond_torsion_3_equivalent(pcff):
    """Simple test of middle_bond_torsion_3 parameters using equivalencies"""
    expected = {
        "V1": "-5.5679",
        "V2": "1.4083",
        "V3": "0.3010",
        "reference": "1",
        "R0": "1.5010",
    }

    i = "h"
    j = "c"
    k = "c5"
    l = "c5"  # noqa: E741
    ptype, key, form, parameters = pcff.middle_bond_torsion_3_parameters(i, j, k, l)
    assert ptype == "equivalent"
    assert key == ("h", "c", "cp", "cp")
    assert parameters == expected



# -*- coding: utf-8 -*-

"""Functions for creating the input for LAMMPS tabulated potentials. """
import numpy as np

import sympy


def tabulate_angle(equation, values, derivative=None, step=0.2):
    """Return the angle, energy and force on a grid of points.

    Some angle terms might go to inifinity at precisely 0.0 degrees, so if the values
    include 'zero-shift' the first angle is set to that value during the calculation,
    but reset to 0.0 in the returned angles.

    Parameters
    ----------
    equation : str
        The symbolic equation as a string.
    values : {str: int or float}
        The values of the constants in the equation, indexex by the name
    derivative : str = None
        The equation for the derivative. If None, it will be derived if possible
    step : float = 0.2
        The step to us for the angle grid, in degrees.
    """
    npts = int(180 / step) + 1
    thetas = np.linspace(0, 180, num=npts)
    if "zero-shift" in values:
        thetas[0] += values.pop("zero-shift")

    symbols = [sympy.symbols(key) for key in values.keys()]
    Theta = sympy.symbols("Theta")
    symbols.append(Theta)
    eqn = sympy.parse_expr(equation)
    E = sympy.lambdify(symbols, eqn, "numpy")

    Es = E(**values, Theta=np.radians(thetas))
    if derivative is None:
        deqn = sympy.diff(eqn, Theta)
    else:
        deqn = sympy.parse_expr(derivative)
    dE = sympy.lambdify(symbols, deqn, "numpy")
    dEs = dE(**values, Theta=np.radians(thetas))

    # Undo any shift
    thetas[0] = 0.0

    # Note angle is degrees. dE is naturally in radians, so convert to degrees
    # Yeah ... it looks like we are converting to radians but that is correct!
    # need to divide by degrees/radian which means multiplying by radians.
    return thetas.tolist(), Es.tolist(), np.radians(dEs).tolist()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `seamm_ff_util.ff_assigner` package."""


def test_methane(pcff_assigner, configuration):
    """Test of atom-type assignment for methane"""
    configuration.from_smiles("C")
    assert pcff_assigner.assign(configuration) == ["c", "hc", "hc", "hc", "hc"]


def test_ethane(pcff_assigner, configuration):
    """Test of atom-type assignment for ethane"""
    configuration.from_smiles("CC")
    assert pcff_assigner.assign(configuration) == [
        "c3",
        "c3",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
    ]


def test_propane(pcff_assigner, configuration):
    """Test of atom-type assignment for propane"""
    configuration.from_smiles("CCC")
    assert pcff_assigner.assign(configuration) == [
        "c3",
        "c2",
        "c3",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
    ]


def test_isobutane(pcff_assigner, configuration):
    """Test of atom-type assignment for isobutane"""
    configuration.from_smiles("CC(C)C")
    assert pcff_assigner.assign(configuration) == [
        "c3",
        "c1",
        "c3",
        "c3",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
    ]


def test_neopentane(pcff_assigner, configuration):
    """Test of atom-type assignment for neopentane"""
    configuration.from_smiles("CC(C)(C)C")
    assert pcff_assigner.assign(configuration) == [
        "c3",
        "c",
        "c3",
        "c3",
        "c3",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
        "hc",
    ]



# -*- coding: utf-8 -*-

"""Top-level package for SEAMM_FF_Util."""

__author__ = """Paul Saxe"""
__email__ = "psaxe@molssi.org"
__version__ = "0.2.1"

from .forcefield import Forcefield  # noqa: F401
from .ff_assigner import FFAssigner  # noqa: F401
from .tabulate import tabulate_angle  # noqa: F401

# Handle versioneer
from ._version import get_versions

versions = get_versions()
__version__ = versions["version"]
__git_revision__ = versions["full-revisionid"]
del get_versions, versions
